#!/bin/bash -e

#
# As the message below says: THIS IS ONLY FOR TESTING
# Get a real certificate for any server you want users to use!
#

echo 'This script will generate all keys and certificates needed for a test setup'
echo '"jfed-rest-api-TESTING-SSL.yml" is configured to use the generated certificates.'
echo '***** Obviously, these certificates are only for testing purposes *****'
echo '* You want a real SSL certificate for the server! Otherwise, your     *'
echo '* users will have to add the self-signed certificate generated below  *'
echo '* to their browser manually.                                          *'

if [ ! -e 'generate-certs.sh' ]
then
    echo 'You need to run this script in the DIR the script is in (generate-certs)'
    exit -1
fi

#URN_TLD="jfed.testing"
URN_TLD="testing.amqp.finterop.eu"

#set -xv
set -x


SOURCE_DIR="${PWD}/"
TARGET_DIR="${PWD}/testing-finterop-AMQP-certificates/"
if [ -e "${TARGET_DIR}" ]
then
    echo 'Target directory '${TARGET_DIR}' already exists: Will no continue.'
    exit -1
fi
mkdir "${TARGET_DIR}"

CA_DIR="${TARGET_DIR}ca/"
mkdir "${CA_DIR}"

cd "${CA_DIR}"
CA_CERTS_DIR="${CA_DIR}certs/"
CA_CRL_DIR="${CA_DIR}crl/"
CA_NEWCERTS_DIR="${CA_DIR}newcerts/"
CA_PRIVATE_DIR="${CA_DIR}private/"
mkdir "${CA_CERTS_DIR}" "${CA_CRL_DIR}" "${CA_NEWCERTS_DIR}" "${CA_PRIVATE_DIR}"
chmod 700 "${CA_PRIVATE_DIR}"
INDEX_FILE="${CA_DIR}index.txt"
touch "${INDEX_FILE}"
SERIAL_FILE="${CA_DIR}serial"
echo 1000 > "${SERIAL_FILE}"

CONFIG_FILE="${CA_DIR}openssl.cnf"
cp -v "${SOURCE_DIR}template_openssl.cnf" "${CONFIG_FILE}"
ESCAPED_CA_DIR=`echo "${CA_DIR}" | sed -e 's/\\//\\\\\\//g'`
sed -i -e "s/#CA_DIR#/${ESCAPED_CA_DIR}/" "${CONFIG_FILE}"
sed -i -e "s/jfed[.]testing/${URN_TLD}/" "${CONFIG_FILE}"

ROOT_KEY_FILE="${CA_PRIVATE_DIR}ca.key.pem"
openssl genrsa -out "${ROOT_KEY_FILE}" 4096
chmod 400 "${ROOT_KEY_FILE}"

ROOT_CERT_FILE="${CA_CERTS_DIR}ca.cert.pem"
cd "${CA_DIR}"
openssl req -config "${CONFIG_FILE}" \
      -key "${ROOT_KEY_FILE}" \
      -new -x509 -days 7300 -sha256 -extensions v3_ca \
      -out "${ROOT_CERT_FILE}"

chmod 444 "${ROOT_CERT_FILE}"

CA_INTERMEDIATE_DIR="${CA_DIR}intermediate/"
mkdir "${CA_INTERMEDIATE_DIR}"
CA_INTERMEDIATE_CERTS_DIR="${CA_INTERMEDIATE_DIR}certs/"
CA_INTERMEDIATE_CRL_DIR="${CA_INTERMEDIATE_DIR}crl/"
CA_INTERMEDIATE_CSR_DIR="${CA_INTERMEDIATE_DIR}csr/"
CA_INTERMEDIATE_NEWCERTS_DIR="${CA_INTERMEDIATE_DIR}newcerts/"
CA_INTERMEDIATE_PRIVATE_DIR="${CA_INTERMEDIATE_DIR}private/"
mkdir "${CA_INTERMEDIATE_CERTS_DIR}" "${CA_INTERMEDIATE_CRL_DIR}" "${CA_INTERMEDIATE_NEWCERTS_DIR}" "${CA_INTERMEDIATE_PRIVATE_DIR}" "${CA_INTERMEDIATE_CSR_DIR}"
chmod 700 "${CA_INTERMEDIATE_PRIVATE_DIR}"
INDEX_FILE="${CA_INTERMEDIATE_DIR}index.txt"
touch "${INDEX_FILE}"
SERIAL_FILE="${CA_INTERMEDIATE_DIR}serial"
echo 1000 > "${SERIAL_FILE}"
echo 1000 > "${CA_INTERMEDIATE_DIR}crlnumber"

INTERMEDIATE_CONFIG_FILE="${CA_INTERMEDIATE_DIR}openssl.cnf"
cp -v "${SOURCE_DIR}template_intermediate_openssl.cnf" "${INTERMEDIATE_CONFIG_FILE}"
ESCAPED_CA_INTERMEDIATE_DIR=`echo "${CA_INTERMEDIATE_DIR}" | sed -e 's/\\//\\\\\\//g'`
sed -i -e "s/#CA_INTERMEDIATE_DIR#/${ESCAPED_CA_INTERMEDIATE_DIR}/" "${INTERMEDIATE_CONFIG_FILE}"
sed -i -e "s/jfed[.]testing/${URN_TLD}/" "${INTERMEDIATE_CONFIG_FILE}"

INTERMEDIATE_KEY_FILE="${CA_INTERMEDIATE_PRIVATE_DIR}intermediate.key.pem"
openssl genrsa \
      -out "${INTERMEDIATE_KEY_FILE}" 4096
chmod 400 "${INTERMEDIATE_KEY_FILE}"

openssl req -config "${INTERMEDIATE_CONFIG_FILE}" -new -sha256 \
      -key "${INTERMEDIATE_KEY_FILE}" \
      -out "${CA_INTERMEDIATE_CSR_DIR}intermediate.csr.pem"
 
INTERMEDIATE_CERT_FILE="${CA_INTERMEDIATE_CERTS_DIR}intermediate.cert.pem"
openssl ca -batch -config "${CONFIG_FILE}" -extensions v3_intermediate_ca \
      -days 3650 -notext -md sha256 \
      -in "${CA_INTERMEDIATE_CSR_DIR}intermediate.csr.pem" \
      -out "${INTERMEDIATE_CERT_FILE}"
chmod 444 "${INTERMEDIATE_CERT_FILE}"

openssl verify -CAfile "${ROOT_CERT_FILE}" \
      "${INTERMEDIATE_CERT_FILE}"

INTERMEDIATE_CERT_CHAIN_FILE="${CA_INTERMEDIATE_CERTS_DIR}ca-chain.cert.pem"
cat "${INTERMEDIATE_CERT_FILE}" "${ROOT_CERT_FILE}" > "${INTERMEDIATE_CERT_CHAIN_FILE}"
chmod 444 "${INTERMEDIATE_CERT_CHAIN_FILE}"

#Generate server key and cert
SERVER_DOMAIN="localhost"
SERVER_KEY_FILE="${CA_INTERMEDIATE_PRIVATE_DIR}${SERVER_DOMAIN}.key.pem"
openssl genrsa \
      -out "${SERVER_KEY_FILE}" 2048
chmod 400 "${SERVER_KEY_FILE}"

SERVER_CONFIG_FILE="${CA_INTERMEDIATE_DIR}server_openssl.cnf"
cp -v "${SOURCE_DIR}template_server_openssl.cnf" "${SERVER_CONFIG_FILE}"
ESCAPED_CA_INTERMEDIATE_DIR=`echo "${CA_INTERMEDIATE_DIR}" | sed -e 's/\\//\\\\\\//g'`
sed -i -e "s/#CA_INTERMEDIATE_DIR#/${ESCAPED_CA_INTERMEDIATE_DIR}/" "${SERVER_CONFIG_FILE}"
sed -i -e "s/jfed[.]testing/${URN_TLD}/" "${SERVER_CONFIG_FILE}"

SERVER_CSR="${CA_INTERMEDIATE_CSR_DIR}${SERVER_DOMAIN}.csr.pem"
openssl req -config "${SERVER_CONFIG_FILE}" \
      -key "${SERVER_KEY_FILE}" \
      -new -sha256 -out "${SERVER_CSR}"

SERVER_CERT_FILE="${CA_INTERMEDIATE_CERTS_DIR}${SERVER_DOMAIN}.cert.pem"
openssl ca -batch -config "${SERVER_CONFIG_FILE}" \
      -extensions server_cert -days 375 -notext -md sha256 \
      -in "${SERVER_CSR}" \
      -out "${SERVER_CERT_FILE}"

openssl verify -CAfile "${INTERMEDIATE_CERT_CHAIN_FILE}" \
      "${SERVER_CERT_FILE}"

#Create user 1 key and cert
for CLIENTID in 1 2 3
do
    CLIENT_KEY_FILE="${CA_INTERMEDIATE_PRIVATE_DIR}client${CLIENTID}.key.pem"
    openssl genrsa \
        -out "${CLIENT_KEY_FILE}" 2048
    chmod 400 "${CLIENT_KEY_FILE}"

    CLIENT_CONFIG_FILE="${CA_INTERMEDIATE_DIR}client${CLIENTID}_openssl.cnf"
    cp -v "${SOURCE_DIR}template_client_openssl.cnf" "${CLIENT_CONFIG_FILE}"
    ESCAPED_CA_INTERMEDIATE_DIR=`echo "${CA_INTERMEDIATE_DIR}" | sed -e 's/\\//\\\\\\//g'`
    sed -i -e "s/#CA_INTERMEDIATE_DIR#/${ESCAPED_CA_INTERMEDIATE_DIR}/" "${CLIENT_CONFIG_FILE}"
    sed -i -e "s/tester@jfed.ilabt.iminds.be/tester${CLIENTID}@jfed.ilabt.iminds.be/" "${CLIENT_CONFIG_FILE}"
    sed -i -e "/commonName.*jFed Testing Client/s/$/ ${CLIENTID}/p" "${CLIENT_CONFIG_FILE}"
    sed -i -e "s/urn:publicid:IDN[+]jfed.testing[+]user[+]tester/urn:publicid:IDN+${URN_TLD}+user+tester${CLIENTID}/" "${CLIENT_CONFIG_FILE}"
    sed -i -e "s/jfed[.]testing/${URN_TLD}/" "${CLIENT_CONFIG_FILE}"

    CLIENT_CSR="${CA_INTERMEDIATE_CSR_DIR}client${CLIENTID}.csr.pem"
    openssl req -config "${CLIENT_CONFIG_FILE}" \
        -key "${CLIENT_KEY_FILE}" \
        -new -sha256 -out "${CLIENT_CSR}"

    CLIENT_CERT_FILE="${CA_INTERMEDIATE_CERTS_DIR}client${CLIENTID}.cert.pem"
    openssl ca -batch -config "${CLIENT_CONFIG_FILE}" \
        -extensions usr_cert -days 375 -notext -md sha256 \
        -in "${CLIENT_CSR}" \
        -out "${CLIENT_CERT_FILE}"

    openssl verify -CAfile "${INTERMEDIATE_CERT_CHAIN_FILE}" \
        "${CLIENT_CERT_FILE}"

    COMBINED_CLIENT_PEM_FILE="${JAVA_DIR}client${CLIENTID}.pem"
    cat "${CLIENT_CERT_FILE}" "${CLIENT_KEY_FILE}" > "${COMBINED_CLIENT_PEM_FILE}"

    echo "Created login PEM file for client ${CLIENTID}: ${COMBINED_CLIENT_PEM_FILE}"
done

#create CRL

INTERMEDIATE_CRL_FILE="${CA_INTERMEDIATE_CRL_DIR}intermediate.crl.pem"
openssl ca -batch -config "${INTERMEDIATE_CONFIG_FILE}" \
      -gencrl -out "${INTERMEDIATE_CRL_FILE}"

openssl crl -in "${INTERMEDIATE_CRL_FILE}" -noout -text



#Generate keystore and truststore in java format
JAVA_DIR="${TARGET_DIR}java/"
mkdir "${JAVA_DIR}"
chmod 700 "${JAVA_DIR}"
JAVA_KEYSTORE_FILE="${JAVA_DIR}testing.keystore"
JAVA_TRUSTSTORE_FILE="${JAVA_DIR}testing.truststore"
JAVA_CRL_FILE="${JAVA_DIR}testing.crl.bundle"

cp "${INTERMEDIATE_CRL_FILE}" "${JAVA_CRL_FILE}"

JAVA_KEYSTORE_PASS="testtest"
JAVA_SERVER_KEY_PASS="testtest"
JAVA_TRUSTSTORE_PASS="testtest"
TMP_PASS="testtest"

TMP_SERVER_KEY_P12="${JAVA_DIR}tmp_server.p12"
openssl pkcs12 -export -in "${SERVER_CERT_FILE}" -inkey "${SERVER_KEY_FILE}" \
    -passout "pass:${TMP_PASS}" \
    -out "${TMP_SERVER_KEY_P12}" -name 'testserver' \
    -CAfile "${INTERMEDIATE_CERT_CHAIN_FILE}" -caname root -chain

keytool -importkeystore -deststorepass "${JAVA_KEYSTORE_PASS}" \
    -destkeypass "${JAVA_SERVER_KEY_PASS}" -destkeystore "${JAVA_KEYSTORE_FILE}" \
    -srckeystore "${TMP_SERVER_KEY_P12}" -srcstoretype 'PKCS12' -srcstorepass "${TMP_PASS}" \
    -alias 'testserver'

rm -f "${TMP_SERVER_KEY_P12}"

keytool -import -alias testca -file "${INTERMEDIATE_CERT_CHAIN_FILE}" \
    -keystore "${JAVA_TRUSTSTORE_FILE}" -storepass "${JAVA_TRUSTSTORE_PASS}" \
    -noprompt

keytool -list -keystore "${JAVA_KEYSTORE_FILE}" -storepass "${JAVA_KEYSTORE_PASS}" 
keytool -list -keystore "${JAVA_TRUSTSTORE_FILE}" -storepass "${JAVA_TRUSTSTORE_PASS}"

